// This is a part of the GWC Library
// Copyright (C) 2001-2002 Aircom software
// All rights reserved.
//
// This source code can be used, distributed or modified
// free of charge only if this header copyright is not removed.
// In other cases, contact us at:
// web:   http://www.aircom.org
// email: info@aircom.org
//

#include "stdafx.h"
#include "GWCColorFunc.h"
#include <math.h>

// Description:
// 
//
// Parameters:
// val - 
//
// Returns:
// 
//===
int Round(double val)
{
	if (fabs(val - (int)val) < 0.5)
		return (int)val;
	else
		return (int)val+(val>0.0 ? 1 : -1);
}

// Description:
// 
//
// Parameters:
// m1 - 
// m2 - 
// h - 
//
// Returns:
// 
//===
double GWCColorFunc::HueToRGB(double m1, double m2, double h)
{
	if (h < 0) h += 1.0;
	if (h > 1) h -= 1.0;
	if (6.0*h < 1.) return (m1+(m2-m1)*h*6.0);
	if (2.0*h < 1.) return m2;
	if (3.0*h < 2.0) return (m1+(m2-m1)*((2.0/3.0)-h)*6.0);
	return m1;
}

// Description:
// 
//
// Parameters:
// H - 
// S - 
// L - 
//
// Returns:
// 
//===
COLORREF GWCColorFunc::HSLtoRGB(double H, double S, double L)
{
	double r, g, b;
	double m1, m2;

	if (S == 0.)
		r = g = b = L;
	else
	{
		if (L <= 0.5)
			m2 = L*(1.0+S);
		else
			m2 = L+S-L*S;

		m1 = 2.0*L-m2;
		r = HueToRGB(m1, m2, H+1.0/3.0);
		g = HueToRGB(m1, m2, H);
		b = HueToRGB(m1, m2, H-1.0/3.0);
	}

	return RGB((BYTE)Round(r*255.), (BYTE)Round(g*255.), (BYTE)Round(b*255.));
}

// Description:
// 
//
// Parameters:
// rgb - 
// H - 
// S - 
// L - 
//===
void GWCColorFunc::RGBtoHSL(COLORREF rgb, double *H, double *S, double *L)
{
	double r = (double)GetRValue(rgb)/255.;
	double g = (double)GetGValue(rgb)/255.;
	double b = (double)GetBValue(rgb)/255.;
	double cmax = max(r, max(g, b));
	double cmin = min(r, min(g, b));

	*L=(cmax+cmin)/2.0;

	if (cmax == cmin) 
	{
		*S = 0;      
		*H = 0; // it's really undefined   
	} 
	else 
	{
		double delta = cmax - cmin;

		if (*L < 0.5)
			*S = delta / (cmax+cmin);      
		else
			*S = delta / (2.0-cmax-cmin);      
		
		if (r == cmax) 
			*H = (g-b) / delta;      
		else if (g == cmax)
			*H = 2.0 + (b-r) / delta;
		else          
			*H = 4.0 + (r-g) / delta;

		*H /= 6.0; 
		if(*H < 0.0)
			*H += 1.;  
	}
}
